'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getDependencyTree = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();
// TODO: This should be exported as a bit component

/**
 * Function for fetching dependency tree of file or dir
 * @param baseDir working directory
 * @param consumerPath
 * @param filePaths path of the file to calculate the dependencies
 * @param bindingPrefix
 * @return {Promise<{missing, tree}>}
 */
var getDependencyTree = exports.getDependencyTree = function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(_ref) {
    var baseDir = _ref.baseDir,
        consumerPath = _ref.consumerPath,
        filePaths = _ref.filePaths,
        bindingPrefix = _ref.bindingPrefix,
        resolveModulesConfig = _ref.resolveModulesConfig;

    var resolveConfigAbsolute, config, result, _groupMissing, groups, foundPackages, tree;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            resolveConfigAbsolute = getResolveConfigAbsolute(consumerPath, resolveModulesConfig);
            config = { baseDir, includeNpm: true, requireConfig: null, webpackConfig: null, visited: {}, nonExistent: [], resolveConfig: resolveConfigAbsolute };
            result = (0, _generateTreeMadge2.default)(filePaths, config);
            _groupMissing = groupMissing(result.skipped, baseDir, consumerPath, bindingPrefix), groups = _groupMissing.groups, foundPackages = _groupMissing.foundPackages;
            tree = groupDependencyTree(result.tree, baseDir, bindingPrefix);
            // Merge manually found packages with madge founded packages

            if (foundPackages && !_ramda2.default.isEmpty(foundPackages)) {
              // Madge found packages so we need to merge them with the manual
              Object.keys(foundPackages).forEach(function (pkg) {
                // locate package in groups(contains missing)
                groups.forEach(function (fileDep) {
                  if (fileDep.packages && fileDep.packages.includes(pkg)) {
                    fileDep.packages = fileDep.packages.filter(function (packageName) {
                      return packageName !== pkg;
                    });
                    (0, _lodash4.default)(tree[fileDep['originFile']], ['packages', pkg], foundPackages[pkg]);
                  }
                });
              });
            }

            updateTreeWithPathMap(tree, result.pathMap);
            return _context.abrupt('return', { missing: groups, tree });

          case 8:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function getDependencyTree(_x) {
    return _ref2.apply(this, arguments);
  };
}();

exports.resolveNodePackage = resolveNodePackage;
exports.resolveModulePath = resolveModulePath;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _ramda = require('ramda');

var _ramda2 = _interopRequireDefault(_ramda);

var _lodash = require('lodash.partition');

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require('lodash.set');

var _lodash4 = _interopRequireDefault(_lodash3);

var _generateTreeMadge = require('./generate-tree-madge');

var _generateTreeMadge2 = _interopRequireDefault(_generateTreeMadge);

var _packageJson = require('../package-json/package-json');

var _packageJson2 = _interopRequireDefault(_packageJson);

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

/**
 * Group dependencies by types (files, bits, packages)
 * @param {any} dependencies list of dependencies paths to group
 * @returns {Function} function which group the dependencies
 */


/**
 * PathMap is used to get the ImportSpecifiers from dependency-tree library
 */
var byType = function byType(list, bindingPrefix) {
  var grouped = _ramda2.default.groupBy(function (item) {
    if (item.includes(`node_modules/${bindingPrefix}`) || item.includes(`node_modules/${_constants.DEFAULT_BINDINGS_PREFIX}`)) return 'bits';
    return item.includes('node_modules') ? 'packages' : 'files';
  });
  return grouped(list);
};

/**
 * Get a path to node package and return the name and version
 *
 * @param {any} packageFullPath full path to the package
 * @returns {Object} name and version of the package
 */
function resolveNodePackage(cwd, packageFullPath) {
  var NODE_MODULES = 'node_modules';
  var result = {};
  // Start by searching in the component dir and up from there
  // If not found search in package dir itself.
  // We are doing this, because the package.json insisde the package dir contain exact version
  // And the component/consumer package.json might contain semver like ^ or ~
  // We want to have this semver as dependency and not the exact version, otherwise it will be considered as modified all the time
  var packageJsonInfo = _packageJson2.default.findPackage(cwd);
  if (packageJsonInfo) {
    // The +1 is for the / after the node_modules, we didn't enter it into the NODE_MODULES const because it makes problems on windows
    var packageRelativePath = packageFullPath.substring(packageFullPath.lastIndexOf(NODE_MODULES) + NODE_MODULES.length + 1, packageFullPath.length);

    var packageName = resolvePackageNameByPath(packageRelativePath);
    var packageNameNormalized = packageName.replace('\\', '/');
    var packageVersion = _ramda2.default.path(['dependencies', packageNameNormalized], packageJsonInfo) || _ramda2.default.path(['devDependencies', packageNameNormalized], packageJsonInfo) || _ramda2.default.path(['peerDependencies', packageNameNormalized], packageJsonInfo);
    if (packageVersion) {
      result[packageNameNormalized] = packageVersion;
      return result;
    }
  }

  // Get the package relative path to the node_modules dir
  var indexOfLastNodeModules = packageFullPath.lastIndexOf(NODE_MODULES) + NODE_MODULES.length + 1;
  var indexOfPackageFolderEnd = packageFullPath.indexOf(_path2.default.sep, indexOfLastNodeModules);
  var packageDir = packageFullPath.substring(0, indexOfPackageFolderEnd);

  // don't propagate here since loading a package.json of another folder and taking the version from it will result wrong version
  // This for example happen in the following case:
  // if you have 2 authored component which one dependet on the other
  // we will look for the package.json on the dependency but won't find it
  // if we propagate we will take the version from the root's package json which has nothing with the component version
  var packageInfo = _packageJson2.default.loadSync(packageDir, false);

  // when running 'bitjs get-dependencies' command, packageInfo is sometimes empty
  // or when using custom-module-resolution it may be empty or the name/version are empty
  if (!packageInfo || !packageInfo.name || !packageInfo.version) return null;
  result[packageInfo.name] = packageInfo.version;
  return result;
}

/**
 * Gets a list of dependencies and group them by types (files, bits, packages)
 * It's also transform the node package dependencies from array of paths to object in this format:
 * {dependencyName: version} (like in package.json)
 *
 * @param {any} list of dependencies paths
 * @param {any} cwd root of working directory (used for node packages version calculation)
 * @returns {Object} object with the dependencies groups
 */
function groupDependencyList(list, cwd, bindingPrefix) {
  var groups = byType(list, bindingPrefix);
  if (groups.packages) {
    var packages = {};
    var unidentifiedPackages = [];
    groups.packages.forEach(function (packagePath) {
      var packageWithVersion = resolveNodePackage(cwd, _path2.default.join(cwd, packagePath));
      if (packageWithVersion) Object.assign(packages, packageWithVersion);else unidentifiedPackages.push(packagePath);
    });
    groups.packages = packages;
    if (!_ramda2.default.isEmpty(unidentifiedPackages)) {
      groups.unidentifiedPackages = unidentifiedPackages;
    }
  }
  return groups;
}

/**
 * Run over each entry in the tree and transform the dependencies from list of paths
 * to object with dependencies types
 *
 * @param {any} tree
 * @param {any} cwd the working directory path
 * @returns new tree with grouped dependencies
 */
function groupDependencyTree(tree, cwd, bindingPrefix) {
  var result = {};
  Object.keys(tree).forEach(function (key) {
    result[key] = groupDependencyList(tree[key], cwd, bindingPrefix);
  });

  return result;
}

/**
 * Get an import statement path to node package and return the package name
 *
 * @param {string} packagePath import statement path
 * @returns {string} name of the package
 */
function resolvePackageNameByPath(packagePath) {
  var packagePathArr = packagePath.split(_path2.default.sep); // TODO: make sure this is working on windows
  // Regular package without path. example - import _ from 'lodash'
  if (packagePathArr.length === 1) return packagePath;
  // Scoped package. example - import getSymbolIterator from '@angular/core/src/util.d.ts';
  if (packagePathArr[0].startsWith('@')) return _path2.default.join(packagePathArr[0], packagePathArr[1]);
  // Regular package with internal path. example import something from 'mypackage/src/util/isString'
  return packagePathArr[0];
}

/**
 * Recursively search for node module inside node_modules dir
 * This function propegate up until it get's to the root provided then stops
 *
 * @param {string} nmPath - package name
 * @param {string} workingDir - dir to start searching of
 * @param {string} root - path to dir to stop the search
 * @returns The resolved path for the package directory
 */
function resolveModulePath(nmPath, workingDir, root) {
  var pathToCheck = _path2.default.resolve(workingDir, 'node_modules', nmPath);

  if (_fs2.default.existsSync(pathToCheck)) {
    return pathToCheck;
  }

  if (workingDir === root) {
    return null;
  }

  var parentWorkingDir = _path2.default.dirname(workingDir);
  if (parentWorkingDir === workingDir) return null;

  return resolveModulePath(nmPath, parentWorkingDir, root);
}

/**
 * Resolve package dependencies from package.json according to package names
 *
 * @param {Object} packageJson
 * @param {string []} packagesNames
 * @returns new object with found and missing
 */
function findPackagesInPackageJson(packageJson, packagesNames) {
  var dependencies = packageJson.dependencies,
      devDependencies = packageJson.devDependencies,
      peerDependencies = packageJson.peerDependencies;

  var foundPackages = {};
  var mergedDependencies = Object.assign({}, dependencies, devDependencies, peerDependencies);
  if (packagesNames && packagesNames.length && !_ramda2.default.isNil(mergedDependencies)) {
    var _partition = (0, _lodash2.default)(packagesNames, function (item) {
      return item in mergedDependencies;
    }),
        _partition2 = _slicedToArray(_partition, 2),
        foundPackagesPartition = _partition2[0],
        missingPackages = _partition2[1];

    foundPackagesPartition.forEach(function (pack) {
      return foundPackages[pack] = mergedDependencies[pack];
    });
    return { foundPackages, missingPackages };
  }
  return { foundPackages: {}, missingPackages: packagesNames };
}
/**
 * Run over each entry in the missing array and transform the missing from list of paths
 * to object with missing types
 *
 * @param {Array} missing
 * @param {string} cwd
 * @param {string} consumerPath
 * @param {string} bindingPrefix
 * @returns new object with grouped missing
 */
function groupMissing(missing, cwd, consumerPath, bindingPrefix) {
  // temporarily disable this functionality since it cause few bugs: explanation below (on using the packageJson)
  // const packageJson = PackageJson.findPackage(cwd);

  /**
   * Group missing dependencies by types (files, bits, packages)
   * @param {Array} missing list of missing paths to group
   * @returns {Function} function which group the dependencies
   */
  var byPathType = _ramda2.default.groupBy(function (item) {
    if (item.startsWith(`${bindingPrefix}/`) || item.startsWith(`${_constants.DEFAULT_BINDINGS_PREFIX}/`)) return 'bits';
    return item.startsWith('.') ? 'files' : 'packages';
  });
  var groups = Object.keys(missing).map(function (key) {
    return Object.assign({ originFile: _path2.default.relative(cwd, key) }, byPathType(missing[key], bindingPrefix));
  });
  groups.forEach(function (group) {
    if (group.packages) group.packages = group.packages.map(resolvePackageNameByPath);
  });
  // This is a hack to solve problems that madge has with packages for type script files
  // It see them as missing even if they are exists
  var foundPackages = {};
  var packageJson = _packageJson2.default.findPackage(cwd);

  groups.forEach(function (group) {
    var missingPackages = [];
    if (group.packages) {
      group.packages.forEach(function (packageName) {
        // Don't add the same package twice
        if (_ramda2.default.contains(packageName, missingPackages)) return;
        var resolvedPath = resolveModulePath(packageName, cwd, consumerPath);
        if (!resolvedPath) {
          return missingPackages.push(packageName);
        }
        var packageWithVersion = resolveNodePackage(cwd, resolvedPath);

        return packageWithVersion ? Object.assign(foundPackages, packageWithVersion) : missingPackages.push(packageWithVersion);
      });
    }
    if (packageJson) {
      var result = findPackagesInPackageJson(packageJson, missingPackages);
      groups.packages = result.missingPackages;
      Object.assign(foundPackages, result.foundPackages);
    }
  });

  // temporarily disable this functionality since it cause this bugs:
  // https://github.com/teambit/bit/issues/635
  // https://github.com/teambit/bit/issues/690


  return { groups, foundPackages };
}

/**
 * if a dependency file is in fact a link file, get its real dependencies.
 */
function getDependenciesFromLinkFileIfExists(dependency, dependencyPathMap) {
  var dependencies = [];
  if (!dependency.importSpecifiers) return dependencies;

  var _loop = function _loop(specifier) {
    var realDep = dependencyPathMap.dependencies.find(function (dep) {
      if (!dep.importSpecifiers) return false;
      return dep.importSpecifiers.find(function (depSpecifier) {
        return depSpecifier.name === specifier.name;
      });
    });
    if (!realDep) {
      // this is not a link file as it doesn't import at least one specifier.
      return 'break';
    }
    var depImportSpecifier = realDep.importSpecifiers.find(function (depSpecifier) {
      return depSpecifier.name === specifier.name;
    });
    var importSpecifier = {
      mainFile: specifier,
      linkFile: depImportSpecifier
    };
    // add to dependencies array
    var file = realDep.relativePath;
    var existingFile = dependencies.find(function (oneDependency) {
      return oneDependency.file === file;
    });
    if (existingFile) {
      existingFile.importSpecifiers.push(importSpecifier);
    } else {
      dependencies.push({ file, importSpecifiers: [importSpecifier] });
    }
  };

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = dependency.importSpecifiers[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var specifier = _step.value;

      var _ret = _loop(specifier);

      if (_ret === 'break') break;
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return dependencies;
}

/**
 * mark dependencies that are link-files as such. Also, add the data of the real dependencies
 */
function updatePathMapWithLinkFilesData(pathMap) {
  pathMap.forEach(function (file) {
    if (!file.dependencies || !file.dependencies.length) return;
    file.dependencies.forEach(function (dependency) {
      if (!dependency.importSpecifiers || !dependency.importSpecifiers.length) {
        // importSpecifiers was not implemented for this language
        return;
      }
      var dependencyPathMap = pathMap.find(function (file) {
        return file.file === dependency.resolvedDep;
      });
      if (!dependencyPathMap || !dependencyPathMap.dependencies || !dependencyPathMap.dependencies.length) return;
      var dependenciesFromLinkFiles = getDependenciesFromLinkFileIfExists(dependency, dependencyPathMap);
      if (dependenciesFromLinkFiles.length) {
        // it is a link file
        dependency.linkFile = true;
        dependency.realDependencies = dependenciesFromLinkFiles;
      }
    });
  });
}

/**
 * remove link-files from the files array and add a new attribute 'linkFiles' to the tree
 */
function updateTreeWithPathMap(tree, pathMap) {
  if (!pathMap || !pathMap.length) return; // tree is empty
  updatePathMapWithLinkFilesData(pathMap);
  Object.keys(tree).forEach(function (mainFile) {
    if (!tree[mainFile].files || !tree[mainFile].files.length) return; // file has no dependency
    var mainFilePathMap = pathMap.find(function (file) {
      return file.relativePath === mainFile;
    });
    if (!mainFilePathMap) throw new Error(`updateTreeWithPathMap: PathMap is missing for ${mainFile}`);
    var files = tree[mainFile].files.map(function (dependency) {
      var dependencyPathMap = mainFilePathMap.dependencies.find(function (file) {
        return file.relativePath === dependency;
      });
      if (!dependencyPathMap) throw new Error(`updateTreeWithPathMap: dependencyPathMap is missing for ${dependency}`);
      var fileObject = {
        file: dependency,
        importSource: dependencyPathMap.importSource,
        isCustomResolveUsed: dependencyPathMap.isCustomResolveUsed
      };
      if (dependencyPathMap.linkFile) {
        fileObject.isLink = true;
        fileObject.linkDependencies = dependencyPathMap.realDependencies;
        return fileObject;
      }
      if (dependencyPathMap.importSpecifiers && dependencyPathMap.importSpecifiers.length) {
        var depImportSpecifiers = dependencyPathMap.importSpecifiers.map(function (importSpecifier) {
          return {
            mainFile: importSpecifier
          };
        });
        fileObject.importSpecifiers = depImportSpecifiers;
      }
      return fileObject;
    });
    tree[mainFile].files = files; // eslint-disable-line no-param-reassign
  });
}

/**
 * config aliases are passed later on to webpack-enhancer and it expects them to have the full path
 */
function getResolveConfigAbsolute(consumerPath, resolveConfig) {
  if (!resolveConfig) return resolveConfig;
  var resolveConfigAbsolute = _ramda2.default.clone(resolveConfig);
  if (resolveConfig.modulesDirectories) {
    resolveConfigAbsolute.modulesDirectories = resolveConfig.modulesDirectories.map(function (moduleDirectory) {
      return _path2.default.isAbsolute(moduleDirectory) ? moduleDirectory : _path2.default.join(consumerPath, moduleDirectory);
    });
  }
  if (resolveConfigAbsolute.aliases) {
    Object.keys(resolveConfigAbsolute.aliases).forEach(function (alias) {
      if (!_path2.default.isAbsolute(resolveConfigAbsolute.aliases[alias])) {
        resolveConfigAbsolute.aliases[alias] = _path2.default.join(consumerPath, resolveConfigAbsolute.aliases[alias]);
      }
    });
  }
  return resolveConfigAbsolute;
}
//# sourceMappingURL=build-tree.js.map